From 2b1f61c89583d7d2b6ad6118325e7669019cf63d Mon Sep 17 00:00:00 2001
From: Masahiko Sawada <sawada.mshk@gmail.com>
Date: Wed, 20 Jul 2022 17:25:11 +0900
Subject: [PATCH v2] Improve description of XLOG_RUNNING_XACTS.

Previously, the description of XLOG_RUNNING_XACTS showed only
top-transaction XIDs and whether subtransactions are overflowed.
This commit improves it to show individual subtransaction XIDs.

This impacts the format of pg_waldump or anything using these
description routines, so no backpatch is done.

Author: Masahiko Sawada <sawada.mshk@gmail.com>
Reviewd by: Fujii Masao <masao.fujii@oss.nttdata.com>
Reviewd by: Kyotaro Horiguchi <horikyota.ntt@gmail.com>
Reviewd by: Ashutosh Bapat <ashutosh.bapat.oss@gmail.com>
Discussion: https://postgr.es/m/CAD21AoAqvaE+XEeXHHPdAGQPcCoGXxuoeutq_nWhUSQvTt5+tA@mail.gmail.com
---
 src/backend/access/rmgrdesc/standbydesc.c | 10 ++++++++--
 1 file changed, 8 insertions(+), 2 deletions(-)

diff --git a/src/backend/access/rmgrdesc/standbydesc.c b/src/backend/access/rmgrdesc/standbydesc.c
index 2dba39e349..a002a50202 100644
--- a/src/backend/access/rmgrdesc/standbydesc.c
+++ b/src/backend/access/rmgrdesc/standbydesc.c
@@ -32,8 +32,14 @@ standby_desc_running_xacts(StringInfo buf, xl_running_xacts *xlrec)
 			appendStringInfo(buf, " %u", xlrec->xids[i]);
 	}
 
-	if (xlrec->subxid_overflow)
-		appendStringInfoString(buf, "; subxid ovf");
+	if (xlrec->subxcnt > 0)
+	{
+		appendStringInfo(buf, "; %d subxacts:", xlrec->subxcnt);
+		for (i = 0; i < xlrec->subxcnt; i++)
+			appendStringInfo(buf, " %u", xlrec->xids[xlrec->xcnt + i]);
+	}
+	else if (xlrec->subxid_overflow)
+		appendStringInfoString(buf, "; subxid overflowed");
 }
 
 void
-- 
2.24.3 (Apple Git-128)

